import {
  Component,
  OnInit,
  ViewChild,
  Inject,
  Input,
  ViewContainerRef,
} from "@angular/core";
import { HttpClient } from "@angular/common/http";
import {
  MatDialogRef,
  MatDialog,
  MAT_DIALOG_DATA,
} from "@angular/material/dialog";
import { MatPaginator } from "@angular/material/paginator";
import { MatTableDataSource } from "@angular/material/table";
import { PageEvent } from "@angular/material/paginator";
import { MatSnackBar, MatSnackBarConfig } from "@angular/material/snack-bar";
import { FormControl } from "@angular/forms";
import { Router, ActivatedRoute } from "@angular/router";
import {
  MatTreeFlatDataSource,
  MatTreeFlattener,
} from "@angular/material/tree";
import { FlatTreeControl } from "@angular/cdk/tree";
import { TreeService } from "../../http-interceptors/tree.service";
import {
  NzFormatEmitEvent,
  NzTreeComponent,
  NzTreeNodeOptions,
} from "ng-zorro-antd/tree";
import { NzModalRef, NzModalService } from "ng-zorro-antd/modal";

@Component({
  selector: "app-examiner-index",
  templateUrl: "./examiner-index.component.html",
  styleUrls: ["./examiner-index.component.scss"],
})
export class ExaminerIndexComponent implements OnInit {
  constructor(
    private router: Router,
    private activatedRoute: ActivatedRoute,
    public http: HttpClient,
    public dialog: MatDialog,
    public snackBar: MatSnackBar,
    private tree: TreeService,
    private modal: NzModalService,
    private viewContainerRef: ViewContainerRef
  ) {}

  //获取登录账号的个人资料
  Profiles: any;
  ngOnInit(): void {
    this.Profiles = JSON.parse(sessionStorage.getItem("creatorData"));
    this.getOrganizations();
    this.getUnittype();
    this.getAlltabledate();
  }

  displayedColumns: string[] = [
    "name",
    "startTime",
    "endTime",
    "examzhong",
    "scc",
  ];
  tabledataSource: any;
  accound;
  helpName; //消防救援对名称
  optionId;
  @ViewChild("son") son;
  //分页
  @ViewChild(MatPaginator, { static: true })
  pageEvent: PageEvent;
  paginator: MatPaginator;
  length: any; //共多少条数据
  pageSize: any = 10; //每页条数
  pageSizeOptions: number[] = [10]; //设置每页条数
  PageNumber: any; //第几页
  startTime: [];
  endTime: [];
  allorganizations: any;
  allunittype: any; //获取所有的单位类型
  jscheck: any; //辖区中队包含下级

  //获得所有组织机构
  getOrganizations() {
    this.http
      .get("/api/Organizations", {
        params: {
          strict: "true",
        },
      })
      .subscribe((data: any) => {
        this.allorganizations = data;
        this.allorganizations.forEach((element) => {
          if (element.id === this.Profiles.organizationId) {
            element.parentId = null;
          }
        });
        this.nodes = [...this.tree.toTree(data)];
        this.orId = this.Profiles.organizationId;
      });
  }
  organizationName: any; //当前单位组织机构名称
  newallorganizations: any; //用于存储在原始数据基础上的每个机构增加children字段
  nodes: any = [];
  expandedKeys: any = []; //默认展开tree节点

  //编辑试卷
  editPaper(item) {
    console.log(item);
    if (item.status == 2) {
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.snackBar.open("考试结束，不能编辑", "确定", config);
    } else if (item.status == 1) {
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.snackBar.open("开考中，不能编辑", "确定", config);
    } else if (item.status == -1) {
      //未发布
      sessionStorage.setItem("paperId", item.id);
      this.router.navigate(["/home/createexam-index/examiner-new-one"]); //跳转试卷列表页面
    } else {
      sessionStorage.setItem("paperId", item.id);
      this.router.navigateByUrl(
        `/examiner/create-test-score?pattern=edit&status=${item.status}`
      );
    }
  }
  lookPaper(item) {
    sessionStorage.setItem("paperId", item.id);
    console.log(item);
    this.router.navigateByUrl(
      `/examiner/create-test-score?pattern=look&status=${item.status}`
    );
  }

  //获得所有单位类型
  getUnittype() {
    this.http.get("/api/BuildingTypes/Simple").subscribe((data) => {
      this.allunittype = data;
    });
  }

  //获得查询支队信息
  help() {
    let paramsdata: any = {
      PageNumber: this.PageNumber || "1",
      PageSize: this.pageSizeOptions[0],
      Sort: null,
      SortType: null,
    };
    //console.log(paramsdata)
    this.http
      .get("/api/Papers", { params: paramsdata })
      .subscribe((data: any) => {
        this.tabledataSource = data;
        console.log(this.tabledataSource);
      });
  }
  //分页事件
  chagePage(e) {
    this.PageNumber = e.pageIndex + 1;
    this.getAlltabledate();
  }
  orId;
  orIdcheck = true;

  //获取表格信息
  getAlltabledate() {
    let paramsdata: any = {
      OrganizationId: this.orId || "",
      HasChildren: this.orIdcheck || "",
      PageNumber: this.PageNumber || "1",
      PageSize: this.pageSizeOptions[0],
      Sort: null,
      SortType: null,
    };
    this.http
      .get("/api/Papers", { params: paramsdata })
      .subscribe((data: any) => {
        this.tabledataSource = data.items;
        this.length = data.totalCount;
      });
  }
  //获取消防救援对信息
  getHelp() {
    let paramsdata: any = {
      Name: "",
      IdentityCard: "",
      Phone: "",
      RealName: "",
      OrganizationId: "",
      PageNumber: "",
      PageSize: "",
      Sort: "",
      RoleType: "1",
      SortType: "",
    };
    this.http
      .get("/api/ExamUsers", { params: paramsdata })
      .subscribe((data: any) => {
        this.helpName = data.items;
      });
  }

  //查询按钮
  findClick() {
    const config = new MatSnackBarConfig();
    config.verticalPosition = "top";
    config.duration = 3000;
    if (!this.orId) {
      this.snackBar.open("请选择消防救援队", "确定", config);
    } else {
      this.PageNumber = 1;
      this.pageEvent.pageIndex = 0;
      this.getAlltabledate();
    }
  }
  //重置按钮
  Reset() {
    this.orId = this.Profiles.organizationId;
    this.orIdcheck = true;
    this.getAlltabledate();
  }
  //删除一套考题
  deleteExam(examid) {
    let isTrue = confirm("您确定要删除吗");
    if (isTrue) {
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.http.delete(`/api/Papers/${examid}`).subscribe((data: any) => {
        this.snackBar.open("删除试卷成功", "确定", config);
        this.getAlltabledate();
      });
    }
  }
  //新增考题跳转
  newExamination() {
    const modal: any = this.modal.create({
      nzTitle: "新增考题",
      nzContent: FinishDia,
      nzViewContainerRef: this.viewContainerRef,
      nzWidth: 688,
      nzFooter: null,
      nzComponentParams: {
        Profiles: this.Profiles,
        treedata: this.nodes,
      },
      nzOnOk: async () => {},
    });
  }
}

//创建试题弹窗
@Component({
  selector: "finish-dialog",
  templateUrl: "finishDia.html",
  styleUrls: ["finishDia.scss"],
})
export class FinishDia {
  // @ViewChild('nzTreSelect', { static: false }) nzTreSelect!: NzTreeComponent;
  constructor(
    private router: Router,
    private http: HttpClient,
    public dialog: MatDialog,
    public dialog2: NzModalRef,
    public snackBar: MatSnackBar,
    private modal: NzModalRef
  ) {}
  //获取登录账号的个人资料
  Profiles: any;
  treedata: any;
  selectedOr;
  nodes = [];
  expandedKeys = [];
  ngOnInit(): void {
    this.Profiles = this.Profiles;
    this.nodes = [...this.treedata];
    this.expandedKeys = [this.treedata[0].key];
  }

  onChange($event: string[]): void {
    console.log($event);
  }

  startTime: string; //考试开始时间
  endTime: string; //考试结束时间
  examName: string; //考试名称
  indexid: string; //创建考试的id
  tabledate;

  //弹窗确定点击事件
  onNoClick(): void {
    const config = new MatSnackBarConfig();
    config.verticalPosition = "top";
    config.duration = 3000;
    if (
      this.startTime != undefined &&
      this.endTime != undefined &&
      this.examName != undefined &&
      this.selectedOr != undefined
    ) {
      if (this.startTime > this.endTime) {
        this.snackBar.open("开始时间不能大于结束时间", "确定", config);
      } else {
        //截取字符串，得到日期部分"2009-12-02",用split把字符串分隔成数组
        var begin1 = this.startTime.substr(0, 10).split("-");
        var end1 = this.endTime.substr(0, 10).split("-");
        //将拆分的数组重新组合，并实例成化新的日期对象
        var date1 = new Date(begin1[1] + -+begin1[2] + -+begin1[0]);
        var date2 = new Date(end1[1] + -+end1[2] + -+end1[0]);
        //得到两个日期之间的差值m，以分钟为单位
        var m = Number(Math.abs(Number(date2) - Number(date1)) / 1000 / 60);
        //小时数和分钟数相加得到总的分钟数
        var min1 =
          parseInt(this.startTime.substr(11, 2)) * 60 +
          parseInt(this.startTime.substr(14, 2));
        var min2 =
          parseInt(this.endTime.substr(11, 2)) * 60 +
          parseInt(this.endTime.substr(14, 2));
        //两个分钟数相减得到时间部分的差值，以分钟为单位
        var n = min2 - min1;
        //将日期和时间两个部分计算出来的差值相加，即得到两个时间相减后的分钟数
        var minutes = m + n;
        let body = {
          id: null,
          title: this.examName,
          duration: minutes,
          modifiedTime: new Date(),
          deleted: false,
          startTime: this.startTime,
          endTime: this.endTime,
          organizationId: this.Profiles.organizationId,
          creatorId: this.Profiles.id,
          paperDataInfo: null,
          organizationIds: this.selectedOr,
        };

        console.log(this.selectedOr);
        this.http.post("/api/Papers", body).subscribe(
          (data) => {
            this.snackBar.open("创建成功", "确定", config);
            this.modal.triggerOk();
            this.tabledate = data;
            console.log(this.tabledate);
            sessionStorage.setItem("paperId", this.tabledate.id);
            this.router.navigate(["/home/createexam-index/examiner-new-one"]); //跳转试卷列表页面
          },
          (err) => {
            this.snackBar.open(err, "确定", config);
          }
        );
      }
    } else {
      if (this.startTime == undefined)
        this.snackBar.open("请输入开始时间", "确定", config);
      else if (this.endTime == undefined)
        this.snackBar.open("请输入结束时间", "确定", config);
      else if (this.examName == undefined)
        this.snackBar.open("请输入试卷名称", "确定", config);
      else if (this.selectedOr == undefined)
        this.snackBar.open("请选择考核队站", "确定", config);
    }
  }

  close() {
    this.dialog2.close();
  }
}
