import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';

@Injectable({
  providedIn: 'root'
})
export class IsLoginService {

  constructor(private http: HttpClient) { }



  isDownloading:boolean = false; //下载状态
  downloadProgress:number = 0;//下载进度


  //下载
  download (e) {
    this.isDownloading = true //开启下载进度条
    let file = e //传递过来的文件元数据
    let fileSize = file.fileLength //下载文件的总大小
    let shardSize = 10 * 1024 * 1024 //文件大小是否大于10MB

    if (file && fileSize<=shardSize) { //<=10MB时直接下载
      
      this.downloadProgress = 70
      this.http.get(`/api/Objects/drives/${file.objectName}`,{responseType: 'blob'},).subscribe(data=>{
        let url = window.URL.createObjectURL(new Blob([data])); //createObjectURL创建一个下载Blob的url地址
        let link = document.createElement("a");
        link.style.display = "none";
        link.href = url;
        link.setAttribute("download", e.fileName);
        document.body.appendChild(link);
        
        link.click();
        this.isDownloading = false //关闭下载进度条
        this.downloadProgress = 0 //初始化进度条
        this.handleData(e.objectName,'下载') //存储用户操作记录
      })
    } else if (file && fileSize>shardSize) { //>10MB时分块下载
      this.blockingDownload(e) //分段下载
    }

  }

  //分段下载并合并
  async blockingDownload (e) {
    let file = e //传递过来的文件元数据
    let fileSize = file.fileLength //下载文件的总大小
    let shardSize = 3 * 1024 * 1024 //3MB一个分片
    let allSlice = Math.ceil(fileSize / shardSize) //总文件/3MB===共分多少段
    let allFile:any = [] //所有的file分段

    for (let i=0;i<allSlice;i++) {
      let start = i * shardSize //每次下载文件开始位置
      let end = Math.min(fileSize, start + shardSize-1); //每次下载文件结束为止

      let result =  await new Promise ((result,reject)=>{
        this.http.get(`/api/Objects/drives/${file.objectName}`,{headers:{'range':`bytes= ${start}-${end}`},responseType:'blob'}).subscribe(data=>{
          result(data) }) 
      })
      allFile.push(result)
      this.downloadProgress = Number((i/allSlice).toFixed(2))*100 //文件进度数

      if (allFile.length === allSlice) { //合并文件输出给浏览器
        let url = window.URL.createObjectURL(new Blob(allFile)); //createObjectURL创建一个下载Blob的url地址
        let link = document.createElement("a");
        link.style.display = "none";
        link.href = url;
        link.setAttribute("download", e.fileName);
        document.body.appendChild(link);

        link.click();
        this.isDownloading = false //关闭下载进度条
        this.downloadProgress = 0 //初始化进度条
        this.handleData(e.objectName,'下载') //存储用户操作记录
      }

    } //for循环结束

  }

  //处理学习记录数据
  handleData (data,operate) {
    let msg = data.split('/')
    let bodyData = {
      creationTime: new Date(), //时间
      operation: operate, //操作
      postName: msg[1], //职务名称
      catalog: msg[2], //目录
      target: data, //objectName
    }
    this.operationLog(bodyData)
  }

  //存储用户操作行为 学习记录
  operationLog (data) {
    this.http.post('/api/StudyRecords',data).subscribe(data=>{})
  }



}
