import { HttpClient } from "@angular/common/http";
import { Component, OnInit, Inject } from "@angular/core";
import { MAT_DIALOG_DATA, MatDialogRef } from "@angular/material/dialog";
import { MatSnackBar, MatSnackBarConfig } from "@angular/material/snack-bar";
import {
  MatTreeFlatDataSource,
  MatTreeFlattener,
} from "@angular/material/tree";
import { FlatTreeControl } from "@angular/cdk/tree";
@Component({
  selector: "app-add-edit-admin",
  templateUrl: "./add-edit-admin.component.html",
  styleUrls: ["./add-edit-admin.component.scss"],
})
export class AddEditAdminComponent implements OnInit {
  constructor(
    private http: HttpClient,
    public dialogRef: MatDialogRef<AddEditAdminComponent>,
    @Inject(MAT_DIALOG_DATA) public data,
    public snackBar: MatSnackBar
  ) {}

  ngOnInit(): void {
    this.dataSource.data = this.data.treeData;
    if (this.data.userData) {
      this.name = JSON.parse(JSON.stringify(this.data.userData.name));
      this.realName = JSON.parse(JSON.stringify(this.data.userData.realName));
      this.organizationId = JSON.parse(
        JSON.stringify(this.data.userData.organizationId)
      );
      this.organizationName = JSON.parse(
        JSON.stringify(this.data.userData.organizationName)
      );
    }
  }

  errmsg: string = null; //捕获错误信息

  name: string = null;
  realName: string = null;
  organizationId: string = null;
  organizationName: string = null;

  private _transformer = (node, level: number) => {
    //初始化tree
    return {
      expandable: !!node.children && node.children.length > 0,
      name: node.name,
      level: level,
      id: node.id,
      parentId: node.parentId,
      children: node.children,
    };
  };
  treeControl = new FlatTreeControl<any>(
    (node) => node.level,
    (node) => node.expandable
  );
  treeFlattener = new MatTreeFlattener(
    this._transformer,
    (node) => node.level,
    (node) => node.expandable,
    (node) => node.children
  );
  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  hasChild = (_: number, node: any) => node.expandable;

  //选择tree节点
  selectTree(e) {
    this.organizationId = e.id;
    this.organizationName = e.name;
  }

  //提交创建表单
  onSubmit(e) {
    if (this.organizationId && this.organizationName) {
      if (this.data.userData) {
        e.phone = this.data.userData.phone;
        e.identityCard = this.data.userData.identityCard;
        e.roleType = 0;
        e.id = this.data.userData.id;
        e.enabled = this.data.userData.enabled;
        e.creationTime = new Date();
        e.organizationId = this.organizationId;
        e.organizationName = this.organizationName;
        this.http
          .put(`/api/ExamUsers/${this.data.userData.id}`, e)
          .subscribe((data) => {
            this.dialogRef.close("success");
          });
      } else {
        e.phone = '13864512542';
        e.identityCard = '370304199809261911';
        e.roleType = 0;
        e.enabled = true;
        e.creationTime = new Date();
        e.organizationId = this.organizationId;
        e.organizationName = this.organizationName;
        this.http.post(`/api/ExamUsers`, e).subscribe((data) => {
          this.dialogRef.close("success");
        });
      }
    } else {
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.snackBar.open("请选择消防救援站", "确定", config);
    }
  }
}
