import { Component, OnInit, ViewChild, Inject } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import {
  MatDialogRef,
  MatDialog,
  MAT_DIALOG_DATA,
} from "@angular/material/dialog";
import { MatPaginator } from "@angular/material/paginator";
import { MatTableDataSource } from "@angular/material/table";
import { PageEvent } from "@angular/material/paginator";
import { MatSnackBar, MatSnackBarConfig } from "@angular/material/snack-bar";
import { FormControl } from "@angular/forms";
import { FlatTreeControl } from "@angular/cdk/tree";
import {
  MatTreeFlatDataSource,
  MatTreeFlattener,
} from "@angular/material/tree";
import { TreeService } from "src/app/http-interceptors/tree.service";
import { AddEditAdminComponent } from "./add-edit-admin/add-edit-admin.component";

@Component({
  selector: "app-administrators",
  templateUrl: "./administrators.component.html",
  styleUrls: ["./administrators.component.scss"],
})
export class AdministratorsComponent implements OnInit {
  constructor(
    public http: HttpClient,
    public dialog: MatDialog,
    public snackBar: MatSnackBar,
    public treeService: TreeService
  ) {}

  loginUserInfo;
  ngOnInit() {
    this.loginUserInfo = JSON.parse(sessionStorage.getItem("creatorData"));
    this.getOrganizations();
  }
  name: any; //用户姓名
  identityCard: any; //用户账号
  organizationId;
  allOrganizations: any; //所有组织机构
  nodes = [];
  //获取所有组织机构
  getOrganizations() {
    this.http
      .get("/api/Organizations", {
        params: {
          strict: "true",
        },
      })
      .subscribe((data: any) => {
        this.allOrganizations = data;
        this.allOrganizations.forEach((element) => {
          if (element.id === this.loginUserInfo.organizationId) {
            element.parentId = null;
          }
        });
        this.nodes = [...this.treeService.toTree(this.allOrganizations)];
        this.organizationId = this.loginUserInfo.organizationId;
        this.getAllUsers();
      });
  }

  displayedColumns: string[] = [
    "identitycard",
    "name",
    "post",
    "time",
    "operation",
  ];

  dataSource: any; //所有企业用户表格数据
  //更新当前页数据
  pageNumber: number = 1; //第几页
  pageSize: any = "10"; //每页条数
  getAllUsers() {
    let data = {
      Name: this.identityCard || "",
      RealName: this.name || "",
      OrganizationId: this.organizationId || "",
      RoleType: "0",
      PageNumber: String(this.pageNumber),
      PageSize: this.pageSize,
    };
    this.http.get("/api/ExamUsers", { params: data }).subscribe((data: any) => {
      this.length = data.totalCount;
      this.pageSize = data.pageSize;
      this.dataSource = new MatTableDataSource<any>(data.items);
    });
  }
  //分页
  @ViewChild(MatPaginator, { static: true })
  pageEvent: PageEvent;
  paginator: MatPaginator;
  length: any; //共多少条数据

  //分页切换
  chagePage(e) {
    this.pageNumber = e.pageIndex + 1;
    this.getAllUsers();
  }

  //清空搜索
  empty() {
    this.pageNumber = 1;
    this.name = "";
    this.identityCard = "";
    this.organizationId = this.loginUserInfo.organizationId;
    this.getAllUsers();
  }

  //创建教员
  open() {
    let data = { treeData: this.nodes, userData: null };
    let dialogRef = this.dialog.open(AddEditAdminComponent, { data });
    dialogRef.afterClosed().subscribe((data) => {
      if (data) {
        const config = new MatSnackBarConfig();
        config.verticalPosition = "top";
        config.duration = 3000;
        this.snackBar.open("创建成功!", "确定", config);
        this.getAllUsers();
      }
    });
  }

  //编辑企业用户
  edit(e) {
    let data = { treeData: this.nodes, userData: e };
    let dialogRef = this.dialog.open(AddEditAdminComponent, { data });
    dialogRef.afterClosed().subscribe((data) => {
      if (data) {
        const config = new MatSnackBarConfig();
        config.verticalPosition = "top";
        config.duration = 3000;
        this.snackBar.open("修改成功!", "确定", config);
        this.getAllUsers();
      }
    });
  }

  //重置密码
  reset(e) {
    this.http.put(`/api/ExamUsers/${e.id}/ResetPassword`, {}).subscribe(
      (data) => {
        const config = new MatSnackBarConfig();
        config.verticalPosition = "top";
        config.duration = 3000;
        this.snackBar.open("重置密码成功!", "确定", config);
      },
      (err) => {
        const config = new MatSnackBarConfig();
        config.verticalPosition = "top";
        config.duration = 3000;
        this.snackBar.open("重置密码失败!", "确定", config);
      }
    );
  }

  //启用
  enabled(e) {
    e.date = new Date();
    e.enabled = true;
    this.http.put(`/api/ExamUsers/${e.id}`, e).subscribe((data) => {
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.snackBar.open("启用成功!", "确定", config);
      this.getAllUsers();
    });
  }

  //禁用
  noEnabled(e) {
    e.date = new Date();
    e.enabled = false;
    this.http.put(`/api/ExamUsers/${e.id}`, e).subscribe((data) => {
      const config = new MatSnackBarConfig();
      config.verticalPosition = "top";
      config.duration = 3000;
      this.snackBar.open("禁用成功!", "确定", config);
      this.getAllUsers();
    });
  }

  //删除
  delete(e) {
    let isTrue = confirm("您确定要删除吗");
    if (isTrue) {
      this.http.delete(`/api/ExamUsers/${e.id}`).subscribe((data) => {
        const config = new MatSnackBarConfig();
        config.verticalPosition = "top";
        config.duration = 3000;
        this.snackBar.open("删除成功!", "确定", config);
        this.getAllUsers();
      });
    }
  }
}
