import { WorkingAreaComponent } from '../working-area.component';
import { GameMode } from './gameMode';
import * as PIXI from 'pixi.js';
import { AxShape } from './axShape';

/**
 * 多点连线
 */
export class MultipointIcon extends AxShape {
    public pointsData: PIXI.Point[];
    public pointsGraphics: PIXI.Sprite[] = [];
    public iconsTilingSprite: PIXI.TilingSprite[] = [];
    style = new PIXI.TextStyle({
        fontFamily: 'Arial',
        fontSize: 18,
        fontStyle: 'normal',
        fontWeight: 'bold',
        fill: ['#000000'],
        stroke: '#ffffff',
        strokeThickness: 3,
        dropShadow: true,
        dropShadowColor: '#000000',
        dropShadowBlur: 3,
        dropShadowAngle: Math.PI / 6,
        dropShadowDistance: 1,
        wordWrap: false,
        wordWrapWidth: 100,
    });

    public text = new PIXI.Text(this.assetData.Name
        + '\r\n'
        + this.assetData.PropertyInfos?.find(item => item.PropertyName === '名称/编号')?.PropertyValue, this.style);
    /**
     *
     * @param texture 图片素材
     * @param points  点集合
     */
    constructor(assetData: any,workingArea: WorkingAreaComponent) {
        super(assetData, workingArea);
        this.name = this.assetData.Id;
        this.pointsData = this.assetData.MultiPoint;
        this.x = this.assetData.Point.x;
        this.y = this.assetData.Point.y;
        this.workingArea.backgroundImage.addChild(this);

        this.text.anchor.set(0.5,0.5);
        // this.text.position = this.getLineCenter(this.pointsData[0], this.pointsData[1]);
        this.text.visible = this.showName;
        this.text.angle = -this.workingArea.backgroundImage.angle;
        this.addChild(this.text);
        this.assetData.Thickness = this.assetData.Thickness === 0 ? 32: this.assetData.Thickness;
        var tileScale = this.assetData.Thickness/64;
        // 画线图标
        for (let i = 0, count = this.pointsData.length - 1; i < count; i++) {
            const pointA = this.pointsData[i];
            const pointB = this.pointsData[i + 1];

            const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
            const a = pointB.x - pointA.x;
            const b = pointB.y - pointA.y;
            const distance = Math.sqrt(a * a + b * b);

            const icon = new PIXI.TilingSprite(PIXI.Texture.from(this.assetData.ImageUrl), distance, this.assetData.Thickness);
            icon.tileScale.set(tileScale);
            icon.anchor.set(0, 0.5);
            icon.x = pointA.x;
            icon.y = pointA.y;
            icon.angle = angle;
            this.iconsTilingSprite.push(icon);
            this.addChild(icon);
            this.text.position = this.getLineCenter(this.pointsData[0], this.pointsData[1]);
            // 等距等分
            this.tileDistanceEqual(icon);
        }
        this.sortableChildren = true;
        this.text.zIndex = this.pointsData.length;
        // 画点
        this.pointsData.forEach((item, index, array) => {
            const iconPoint = PIXI.Sprite.from(this.pointTexture);
            iconPoint.anchor.set(0.5);
            iconPoint.x = item.x;
            iconPoint.y = item.y;
            iconPoint.visible = false;
            this.pointsGraphics.push(iconPoint);
            this.addChild(iconPoint);
        });
        // 添加圆点事件
        this.pointsGraphics.forEach((item, index, array) => {
            item.interactive = true;
            item.on('pointerdown', event => {
                event.stopPropagation();
                if (this.workingArea.allowEdit && this.assetData.GameMode === this.workingArea.canvasData.gameMode) {
                    event.currentTarget.data = event.data;
                    event.currentTarget.alpha = 0.5;
                    event.currentTarget.dragging = true;
                }
            })
                .on('pointerup', event => {
                    if (event.currentTarget.dragging) {
                        event.currentTarget.alpha = 1;
                        event.currentTarget.dragging = false;
                        event.currentTarget.data = null;
                    }
                })
                .on('pointerupoutside', event => {
                    if (event.currentTarget.dragging) {
                        event.currentTarget.alpha = 1;
                        event.currentTarget.dragging = false;
                        event.currentTarget.data = null;
                    }
                })
                .on('pointermove', event => {
                    if (event.currentTarget.dragging) {
                        const newPosition = event.currentTarget.data.getLocalPosition(event.currentTarget.parent);
                        event.currentTarget.x = newPosition.x;
                        event.currentTarget.y = newPosition.y;

                        this.assetData.MultiPoint[index].x = newPosition.x;
                        this.assetData.MultiPoint[index].y = newPosition.y;
                        this.workingArea.canvasData.isChange = true;

                        if (index === 0) {// 第一个点
                            this.iconsTilingSprite[index].x = newPosition.x;
                            this.iconsTilingSprite[index].y = newPosition.y;

                            const pointA = array[index];
                            const pointB = array[index + 1];

                            const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
                            const a = pointB.x - pointA.x;
                            const b = pointB.y - pointA.y;
                            const distance = Math.sqrt(a * a + b * b);
                            this.iconsTilingSprite[index].angle = angle;
                            this.iconsTilingSprite[index].width = distance;

                            this.tileDistanceEqual(this.iconsTilingSprite[index]);
                        } else if (index < array.length - 1) {// 不是第一个点，也不是最后一个点
                            this.iconsTilingSprite[index].x = newPosition.x;
                            this.iconsTilingSprite[index].y = newPosition.y;

                            const pointA = array[index]; // 当前点
                            const pointB = array[index + 1]; // 后一个点
                            const pointC = array[index - 1]; // 前一个点

                            const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
                            const a = pointB.x - pointA.x;
                            const b = pointB.y - pointA.y;
                            const distance = Math.sqrt(a * a + b * b);
                            this.iconsTilingSprite[index].angle = angle;
                            this.iconsTilingSprite[index].width = distance;

                            this.tileDistanceEqual(this.iconsTilingSprite[index]);

                            const angleC = Math.atan2((pointA.y - pointC.y), (pointA.x - pointC.x)) * (180 / Math.PI);
                            const aC = pointA.x - pointC.x;
                            const bC = pointA.y - pointC.y;
                            const distanceC = Math.sqrt(aC * aC + bC * bC);
                            this.iconsTilingSprite[index - 1].angle = angleC;
                            this.iconsTilingSprite[index - 1].width = distanceC;

                            this.tileDistanceEqual(this.iconsTilingSprite[index - 1]);
                        } else if (index === array.length - 1) { // 最后一个点
                            const pointA = array[index]; // 当前点
                            const pointC = array[index - 1]; // 前一个点

                            const angleC = Math.atan2((pointA.y - pointC.y), (pointA.x - pointC.x)) * (180 / Math.PI);
                            const aC = pointA.x - pointC.x;
                            const bC = pointA.y - pointC.y;
                            const distanceC = Math.sqrt(aC * aC + bC * bC);
                            this.iconsTilingSprite[index - 1].angle = angleC;
                            this.iconsTilingSprite[index - 1].width = distanceC;

                            this.tileDistanceEqual(this.iconsTilingSprite[index - 1]);
                        }
                        this.drawBorder(1 / this.workingArea.backgroundImage.scale.x);
                        this.text.position = this.getLineCenter(this.pointsData[0], this.pointsData[1]);
                    }
                })
                .on('rightclick', event => {
                });
        });
    }
    /**
     * 设置点显示状态
     * @param value 显示状态
     */
    public setPointVisiable(value: boolean) {
        this.pointsGraphics.forEach((item) => {
            item.visible = value;
        });
    }
    /**
     * 
     * @param scale 绘制边框
     */
    public drawBorder(scale: number) {
        let visible = this.pointsGraphics[0].visible;
        this.setPointVisiable(false);        
        super.drawBorder(scale);
        this.setPointVisiable(visible);
    }
    // 设置名称
    public setNameVisible(value: boolean, mode: GameMode) {
        if (this.assetData.GameMode === mode) {
            this.text.visible = value;
        }
    }
    // 设置缩放
    public setItemScale(scale: number) {
        // this.text.scale.set(scale);
        this.pointsGraphics.forEach((item, index, array) => {
            item.scale.set(scale);
        });
    }
    // 刷新数据
    public refresh() {
        this.assetData.Thickness = this.assetData.Thickness === 0 ? 32: this.assetData.Thickness;
        this.iconsTilingSprite.forEach(item => {   
            // 厚度优化
            this.tileDistanceEqual(item);
        });
        this.text.text = this.assetData.Name
            + '\r\n'
            + this.assetData.PropertyInfos.find(item => item.PropertyName === '名称/编号')?.PropertyValue;
        this.text.angle = -this.workingArea.backgroundImage.angle;
    }

    // 等距等分
    public tileDistanceEqual(icon: PIXI.TilingSprite) {
        icon.height = this.assetData.Thickness;
        var tileScale = this.assetData.Thickness / 64;
        icon.tileScale.set(tileScale);
        console.log(tileScale);
        var tileWidth = tileScale * 64;
        var tileCount = Math.ceil(icon.width / tileWidth);
        tileWidth = icon.width / tileCount;
        tileScale = tileWidth / 64;
        console.log(tileScale);
        icon.tileScale.set(tileScale);
    }
}
